/**
 * apeman app to recover password.
 * @memberof module:apeman-app-recover/lib
 * @function create
 * @param {object} UserRecover - RecoverModel.
 * @param {object} [options] - Optional settings.
 * @param {number} [options.expireDuration=ONE_DAY] - Expire duration
 * @param {function} [options.beforeSend=noop()] - Handle before send
 * @param {function} [options.beforeReset=noop()] - Handle before reset
 * @param {function} [options.afterSend=noop()] - Handle after send
 * @param {function} [options.afterReset=noop()] - Handle after reset
 * @param {object} [options.sendSchema=defaultSchema('send')] - Schema for send.
 * @param {object} [options.resetSchema=defaultSchema('send')] - Schema for reset.
 * @param {number} [options.passwordSpec='pdkdf2:120:84'] - Password algorithm, iterations, length.
 * @returns {function} - Defined app function.
 *
 */

"use strict";

const argx = require('argx'),
    objnest = require('objnest'),
    assert = require('assert'),
    defineDigest = require('apeman-app-sign/lib/digesting/define_digest'),
    route = require('apeman-app-route'),
    validate = require('apeman-app-validate'),
    noop = require('apeman-app-noop'),
    combine = require('apeman-app-combine'),
    action = require('apeman-app-action'),
    apemanerror = require('apemanerror'),
    doSend = require('./recovering/do_send'),
    doReset = require('./recovering/do_reset'),
    schemas = require('./schemas');

const debug = require('debug')('apeman:app:recover');

const ONE_DAY = 24 * 60 * 60 * 1000;

/** @lends create */
function create(UserRecover, options) {
    let args = argx(arguments);
    options = objnest.expand(args.pop('object') || {});

    let digest = defineDigest(options.passwordSpec);

    let expireDuration = options.expireDuration || ONE_DAY,
        beforeSend = options.beforeSend || noop(),
        beforeReset = options.beforeReset || noop(),
        afterSend = options.afterSend || noop(),
        afterReset = options.afterReset || noop()
        ;

    function sendResult(req, res) {
        let format = req.format || options.format,
            result = req.result;
        res.send(result, format);
    }

    function handleError(err, req, res, next) {
        let format = req.format || options.format;
        let sequelizeError = apemanerror.sequelizeError(err.errors);
        if (sequelizeError) {
            res.statusCode = 400;
            res.send({errors: sequelizeError}, format);
            return;
        }
        next(err);
    }

    let handlers = [
        (req, res, next) => {
            next();
        },
        route({
            '/send': {
                'POST': [
                    beforeSend,
                    validate(options.sendSchema || schemas('send')),
                    action('recover-send'),
                    function recoverSend(req, res, next) {
                        let email = req.body.email;
                        doSend(UserRecover, email, expireDuration)
                            .then(recover => {
                                let success = !!recover;
                                if (success) {
                                    req.result = {success, recover};
                                } else {
                                    res.statusCode = 404;
                                    req.result = {};
                                }
                                next();
                                return recover;
                            })
                            .catch(err => {
                                handleError(err, req, res, next);
                                return err;
                            });

                    },
                    afterSend,
                    sendResult
                ]
            },
            '/reset': {
                'POST': [
                    beforeReset,
                    validate(options.resetSchema || schemas('reset')),
                    action('recover-reset'),
                    function recoverReset(req, res, next) {
                        let token = req.body.token,
                            password = req.body.password;
                        doReset(UserRecover, digest, token, password)
                            .then(recover => {
                                let success = !!recover;
                                if (success) {
                                    req.result = {success, recover};
                                } else {
                                    res.statusCode = 404;
                                    req.result = {};
                                }
                                next();
                                return recover;
                            })
                            .catch(err => {
                                handleError(err, req, res, next);
                                return err;
                            });
                    },
                    afterReset,
                    sendResult
                ]
            }
        })
    ];

    /**
     * Defined app.
     * @function app
     * @param {object} req - Request object.
     * @param {object} res - Response object.
     * @param {function} next - Pass to next handler.
     */
    function app(req, res, next) {
        assert(req.body, 'req.body not found. Use "apeman-app-form" before this.');
        assert(res.header, 'req.header not found. Use "apeman-app-header" before this.');
        assert(res.send, 'req.send not found. Use "apeman-app-send" before this.');

        app.$handler(req, res, next);
    }

    return Object.assign(app, {
        $handler: combine(handlers),
        $endpoints: handlers[1].$endpoints,
        $desc: "Define recover endpoints"
    });
}

module.exports = create;
